/*
 * CMultiSkelWindow.cp
 *
 * This is an intermediate class between CWindow and all
 * the windows in MultiSkel++.  In the MultiSkel demo, the
 * methods defined here were declared as C functions which were
 * accessible in all MultiSkel source files.  Here, they are declared
 * as protected methods, so they are hidden from outside use,
 * but not from any subclass of this class.
 *
 *
 * Modified by: fadushin 1/2/96
 * Uses: TransSkel++ 3.00
 */
#include "CMultiSkelWindow.h"


/*
 * Miscellaneous routines
 * These take care of drawing the grow box and the line along
 * the right edge of the window, and of setting and resetting the clip
 * region to disallow drawing in that right edge by the other drawing
 * routines.
 */


void
CMultiSkelWindow::DrawGrowBox(void)
{
Rect		r;
RgnHandle	oldClip;

	r = wind->portRect;
	r.left = r.right - 15;		/* draw only along right edge */
	oldClip = NewRgn ();
	GetClip (oldClip);
	ClipRect (&r);
	DrawGrowIcon (wind);
	SetClip (oldClip);
	DisposeRgn (oldClip);
}


void
CMultiSkelWindow::SetWindClip(void)
{
Rect		r;

	r = wind->portRect;
	r.right -= 15;		/* don't draw along right edge */
	oldClip = NewRgn ();
	GetClip (oldClip);
	ClipRect (&r);
}


void
CMultiSkelWindow::ResetWindClip (void)
{
	SetClip (oldClip);
	DisposeRgn (oldClip);
}


/*
 * Show a window if it's not visible.  Select the window FIRST, then
 * show it, so that it comes up in front.  Otherwise it will be drawn
 * in back then brought to the front, which is ugly.
 *
 * The test for visibility must be done carefully:  the window manager
 * stores 255 and 0 for true and false, not real boolean values.
 */

void
CMultiSkelWindow::doShow(void)
{

	if (((WindowPeek) wind)->visible == false)
	{
		SelectWindow (wind);
		ShowWindow (wind);
	}
}
