/*
 * TransSkel++ CWindow subclass demonstration: CRegionWindow
 *
 * This class is based entirely on Paul Dubois' RegionWindow module of 12 Jan 91:
 *
 * "This module handles a window in which the mouse may be clicked and
 * dragged to draw rectangles.  The rects so drawn are combined into
 * a single region, the outline of which is drawn.  Rects drawn while
 * the shift key is held down are subtracted from the region.
 * Double-clicking the mouse clears the display.  If the window is
 * resized, the region that is drawn is resized as well.
 *
 * BUG: should measure double-click time from mouse-up to next mouse down,
 * not time between mouse-downs."
 *
 * Modified by: fadushin 1/2/96
 * Uses: TransSkel++ 3.00
 */

#include "TransSkel++.h"
#include "CRegionWindow.h"
#include "MultiSkel++.h"



void
CRegionWindow::MarqueeRgn (RgnHandle r)
{
PenState	p;
Byte		b;
short		i;

	GetPenState (&p);
	PenPat ((ConstPatternParam) &marqueePat);
	PenMode (patCopy);
	FrameRgn (r);
	SetPenState (&p);
	b = ((char *) &marqueePat)[0];		/* shift pattern for next call */
	for (i = 0; i < 7; ++i)
		((char *) &marqueePat)[i] = ((char *) &marqueePat)[i+1];
	((char *) &marqueePat)[7] = b;
}


/*
 * While mouse is down, draw gray selection rectangle in the current
 * port.  Return the resultant rect in dstRect.  The rect is always
 * clipped to the current portRect.
 */

void
CRegionWindow::doSelectRect(Point startPoint, Rect *dstRect)
{
Point		pt, dragPt;
Rect		rClip;
GrafPtr		port;
Boolean		result;
PenState	ps;
short		i;

	GetPort (&port);
	rClip = port->portRect;
	rClip.right -= 15;
	GetPenState (&ps);
	PenPat ((ConstPatternParam) &qd.gray);
	PenMode (patXor);
	dragPt = startPoint;
	Pt2Rect (dragPt, dragPt, dstRect);
	FrameRect (dstRect);
	for (;;)
	{
		GetMouse (&pt);
		if (!EqualPt (pt, dragPt))	/* mouse has moved, change region */
		{
			FrameRect (dstRect);
			dragPt = pt;
			Pt2Rect (dragPt, startPoint, dstRect);
			result = SectRect (dstRect, &rClip, dstRect);
			FrameRect (dstRect);
			for (i = 0; i < 1000; ++i) { /* empty */ }
		}
		if (!StillDown ()) break;
	}
	FrameRect (dstRect);	/* erase last rect */
	SetPenState (&ps);
}


/*
 * On double-click, clear window.  On single click, draw gray selection
 * rectangle as long as mouse is held down.  If user draws non-empty rect,
 * then add it to the selection region and redraw the region's outline.
 * If the shift-key was down, then subtract the selection region instead
 * and redraw.
 */

void
CRegionWindow::doMouseClick(Point pt, long t, short mods)
{
Rect		r;
RgnHandle	rgn;

	r = wind->portRect;
	if (pt.h >= r.right - 15)		/* must not click in right edge */
		return;
	if (t - selectWhen <= GetDblTime ())	/* it's a double-click */
	{
		selectWhen = 0L;		/* don't take next click as double-click */
		SetWindClip ();
		EraseRgn (selectRgn);
		ResetWindClip ();
		SetEmptyRgn (selectRgn);	/* clear region */
	}
	else
	{
		selectWhen = t;				/* update click variables */
		selectWhere = pt;
		doSelectRect (pt, &r);	/* draw selection rectangle */
		if (!EmptyRect (&r))
		{
			EraseRgn (selectRgn);
			selectWhen = 0L;
			rgn = NewRgn ();
			RectRgn (rgn, &r);
			if ((mods & shiftKey) != 0)		/* test shift key */
				DiffRgn (selectRgn, rgn, selectRgn);
			else
				UnionRgn (selectRgn, rgn, selectRgn);
			DisposeRgn (rgn);
		}
	}
}


void
CRegionWindow::doIdle (void)
{
short	i;

	SetWindClip ();
	MarqueeRgn (selectRgn);	/* draw selection region outline */
	ResetWindClip ();		/* restore previous clipping */
}


/*
 * Redraw the current region.  If the window was resized, resize
 * the region to fit.
 */

void
CRegionWindow::doUpdate (Boolean resized)
{
Rect	r;

	r = wind->portRect;
	EraseRect (&r);
	if (resized)
	{
		rgnPortRect.right -= 15;	/* don't use right edge of window */
		r.right -= 15;
		MapRgn (selectRgn, &rgnPortRect, &r);
		rgnPortRect = wind->portRect;
	}
	DrawGrowBox ();
	doIdle ();
}


void
CRegionWindow::doActivate (Boolean active)
{
	DrawGrowBox ();
//	if (active)
//		DisableItem (editMenu, 0);
//	else
//		EnableItem (editMenu, 0);
//	DrawMenuBar ();
}


CRegionWindow::~CRegionWindow(void)
{
	DisposeRgn (selectRgn);
}



CRegionWindow::CRegionWindow(short resID)
              :CMultiSkelWindow(resID, STORE_IN_HEAP, BRING_TO_FRONT, false){
	rgnPortRect = wind->portRect;
	selectRgn = NewRgn ();	/* selected region empty initially */
	selectWhen = 0L;		/* first click can't be taken as dbl-click */
	
	//marqueePat[0] = 0x0f;
	//marqueePat[1] = 0x87;
	//marqueePat[2] = 0xc3;
	//marqueePat[3] = 0xe1;
	//marqueePat[4] = 0xf0;
	//marqueePat[5] = 0x78;
	//marqueePat[6] = 0x3c;
	//marqueePat[7] = 0x1e;
}
