/*
 * MultiSkel++ - TransSkel++ multiple-window demonstration main module
 *
 * Based on Paul Dubois' MultiSkel TransSkel demo.  From MultiSkel:
 *
 *    "This module performs setup and termination operations, installs
 *    the window and menu handlers, and processes menu item selections.
 *
 *    There are four window handlers in this demonstration.  The code
 *    for each handler is in its own module.
 *
 *    Help Window		Scrollable non-editable text window
 *    Edit Window		Non-scrollable editable text window
 *    Zoom Window		Non-manipulable graphics display window
 *    Region Window	Manipulable graphics display window"
 *
 * MultiSkel++ demonstrates the use of object inheritance
 * with 2 the types of window:
 *
 * CZoomWindow
 * CRegionWindow
 *
 * A base class CMultiSkelWindow is defined which contains member
 * function all of the above types of window share.
 *
 * fadushin 1/2/96
 * Uses TransSkel++ 3.00
 */

#include "TransSkel++.h"
#include "MultiSkel++.h"
#include "CZoomWindow.h"
#include "CRegionWindow.h"



/*
 *  The Apple Menu
 */

/*
 * Handle selection of About MultiSkel... item from Apple menu
 */

void
CMultiSkelAppleMenu::doSelect(short item)
{
	(void) SkelAlert (aboutAlrtRes, SkelDlogFilter (nil, true),
										skelPositionOnParentDevice);
	SkelRmveDlogFilter ();
}



/*
 * The File Menu
 */

/* File menu item numbers */
typedef enum {
	open = 1,
	close,
	/* --- */
	quit = 4
} fileItems;


/*
 * Process selection from File menu.
 *
 * Open		Make all four windows visible
 * Close	Hide the frontmost window.  If it belongs to a desk accessory,
 *			close the accessory.
 * Quit		Request a halt by calling SkelHalt().  This makes SkelMain
 *			return.
 */

void
CFileMenu::doSelect(short item)
{
	switch (item){
		case open:
		    gApp->zoomWindow->doShow();
		    gApp->regionWindow->doShow();
			break;
        case close:
			SkelClose (FrontWindow ()); // Safe--if FrontWindow is a CWindow, its
			                            // doClose method will be called.
			break;
        case quit:
			SkelStopEventLoop ();		/* request halt */
			break;
	    }
}




/* a global pointer to the application */
MultiSkelApp *gApp;

/*
 * The MultiSkel Application constructor
 */
MultiSkelApp::MultiSkelApp(void)
             :CApplication(nil)
{
   long f, b;
   
   SkelGetWaitTimes (&f, &b);
   SkelSetWaitTimes (f, b);
   
   zoomWindow   = new CZoomWindow(zoomWindRes);
   regionWindow = new CRegionWindow(rgnWindRes);
}

/*
 * The MultiSkel Application destructor
 */
MultiSkelApp::~MultiSkelApp(void)
{
   delete zoomWindow;
   delete regionWindow;
}

/*
 * The main cheese
 */
int
main (void)
{
   MultiSkelApp a;
   
   gApp = &a;
   
   a.Run();
   	
   return(0);
}
